/*! \file error_base.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief Anything related to general error handling.
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef ERROR_BASE_H
#define ERROR_BASE_H


#ifdef __HAVE_CONFIG_H__
#include "../config.h"
#else
#define METH_NAME (__FILE__ ":")
#endif

/*!\def METH_NAME

 \brief use this as argument \a sF at the  Error::* constructors
*/
/*!\def DEBUG
 
 \brief Set the debug flag to enable some debug output and extra checkings.
*/
/*!\def DGPR(name)
 \brief Debug printing

 This macro shows verbous the postion, where you called it, the arguments name
 and its value.
 Its defintion is void, if the \a DEBUG flag is not set.
*/
#ifdef DEBUG
#include <iostream>
#define DGPR(name) {\
 std::cerr << __FILE__ << ": " << __LINE__ << ": " << METH_NAME << ": " <<\
 #name << "=>" << name << "<=" << std::endl;\
}
#else
#define DGPR(name) 
#endif

#include <string>

//! Classes to throw arround in case of an error
namespace Error{

/*!\brief Base class to all erorrs.

 Do not throw this class. But it is usefull to catch any kind of error.
*/
struct base{
 virtual ~base(){};

 std::string sFunc, //!< code position of error
             sWhat; //!< error description
 /*!\brief Construct an error
  \param fu_name Code position of error (ususally \a METH_NAME).
  \param desc Error description.
 */ 
 base(const std::string& fu_name,const std::string& desc):
  sFunc(fu_name),sWhat(desc){};
 //! Get an pretty error string.
 virtual operator std::string(){return std::string("base error in ")+sFunc+": "
  +sWhat;}
};

/*!\brief Throw this error if a situation occured, that indicates a bug
 
 This can be used for extra error checking or if you want to catch illegal
 method calls ...
*/
struct Program:base{ 
 virtual ~Program(){};

 /*!\brief Construct an program error.
  \param fu_name Code position of error (ususally \a METH_NAME).
  \param desc Error description.
 */ 
 Program(const std::string& fu_name,const std::string& desc):
  base(fu_name,desc){};
 //! Get an pretty error string.
 virtual operator std::string(){return "programm error in "+sFunc+": "+sWhat;}
};

/*!\brief Thow this error, if a runtime error occured
 
 You may use this class at all normal error situations. It just indicates, that
 some runtime problem occured, such as an error on opening a file.
*/
struct Runtime:base{ 
 virtual ~Runtime(){};

 /*!\brief Construct an runtime error.
  \param fu_name Code position of error (ususally \a METH_NAME).
  \param desc Error description.
 */ 
 Runtime(const std::string& fu_name,const std::string& desc):
  base(fu_name,desc){};
 //! Get an pretty error string.
 virtual operator std::string(){return "runtime error in "+sFunc+": "+sWhat;}
};
}
#endif
