/*! \file vtql_base.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief Bases classes to get a connection to a VTQLserver
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>


#ifndef VTQL_BASE_H
#define VTQL_BASE_H

#include "sock.h"
#include "thread_util.h"
#include <string>
#include <vector>

extern "C" {
 #include "extern/vt.h"
}
#undef W
#undef H

/*!\brief Contains all high level stuff to handle connections to VTQLserver.

\todo Exception documentation.
*/
namespace VTQL{
//! Version number of library. It is "major.minor.rev (day.month.year)"
extern const std::string& sVersion;

/*!\brief Provides basic stuff for connections to a VTQLserver.
 
 Usually you will not use this abstract class, but higher level ones. You can
 simpliy connect to the server and send lines to. There are tools to read lines
 and the idea is to call callback classes on on read lines. The method \a
 Process(bool,bool) that is intended to read and call back is pure.
*/
class Connect{ 
  public: virtual ~Connect(){};
  /*! \warning The copy constructor is not implemented. */
  Connect(const Connect&);

  /*! \brief Open a connection to a server.
   
   \param file_name File name of a socket of VTQLserver.
  */  
  Connect(const std::string& file_name);
  /*!\brief Open a connection to a server (advanced).

   If you have an file descriptor and it is connected to a server, you can use
   this constructor.
   \param file_descriptor A file descriptor that is connected to a VTQLserver.
   \sa sock::Socket_basic::Socket_basic(int), sock::Socket_basic::operator int()
  */
  Connect(const sock::Socket_basic& file_descriptor);

  /*!\brief Send a line to server.
    
   The input string+newline is added to the output queue. Send this queue with
   the Poll(bool) method to the server.
   \param line The line, that should be sent.
   \return The calling instance itself.
  */ 
  virtual Connect& operator <<(const std::string& line);

  /*!\brief Read form server and handle the lines read.

   This method gets called by CModul::Wait(bool) and should read all data,
   that is in the input queue, i.e. until InpLineC() is \a 0. The CModul
   expects this method to call the callback classes with the data just read.
   It must not contain a loop to read until the command output had finished,
   but its return value should indicate a CModul, that it can return with true
   from the CModul::Wait(bool) method.
   It is expectet, that this function calls the Poll(bool) method.

   \param block true: blocking ; false: nonblocking i/o
   \retrun true if command output has finished, false otherwise.
   \sa CModul::Wait(bool)
  */
  virtual bool Process(bool block)=0;

  /*@{
   \name Callback Interface
   
   A CModul adds itself to this list and hopes to get a callback by the
   Process(bool) method.
  */
  //! we use a pair of \a int and \a std::string for a callback
  typedef thread::CallBack<std::pair<int,std::string> > CallB; 
  CallB cb; //!< the callback structure is public
  //@}
 protected:
  //! \brief Get the number of input lines in buffer
   size_t InpLineC(){return Inp.Lines();}
   /*!\brief get a line (without newline) from read buffer

   You can get the number of input lines with InpLineC. Fill the buffer with
   Poll(bool).
  */
  std::string FetchLine();
  /*!\brief Flushes i/o buffers.

   Tries to write and read a bit from/to buffers (it does not contain a loop).
   \param b true: waiting for a chance to read and/or write
  */
  void Poll(bool block);
private:
  sock::Socket_basic sCon;
  sock::PollClass poll;
  sock::Output Outp;
  sock::Input Inp;
};

/*!\brief Base class for all moduls.
 
 This class hides all the wired things of \a Connect, such as the Callback
 interface and the Connect::Process(bool) method. Its whole lifetime it is
 in the callback list, i.e. it gets called on every input line the method
 Connect::Process(bool) emits. 
*/
class CModul:public Connect::CallB::CBFunct{
 public:
  /*!\brief Removes the modul form callback list of the connection.*/
  virtual ~CModul(){StopCB();}
  /*!\warning The copy constructer is not implemented. */
  CModul(const CModul&);

  /*!\brief Inserts this modul into the callback list. 
   \param connection The modul does all the i/o with this connection */
  CModul(Connect& connection):qc(connection),iCB_ID(0){StartCB();}
  /*!\brief Wait for (some kind of) completion of the command.
   
   You can either wait (i.e. blocking) or just check for completion
   (i.e. nonblocking).

   \param block false: nonblocking ; true blocking
   \return true if command was completed, false otherwise
  */
  virtual bool Wait(bool block=true);

  /*!\brief this function gets called at every server output line (callback).

   In detail, the Connection::Process(bool) funtion should call back it,
   unless you did not remove it from the callback list with StopCB().

   The hard thing is to find out which are the output lines, you need.
   It is a good idea to exclude yourself from callback if the command output
   has been recieved via the StopCB() method.

   Use the class CQModul for queries and configuration commands and use the
   class CEModul for event commands, since they solve a lot of trouble you may
   have with this base class :-)
   

   \param p The \a Arg type is std::pair<int,std::string>, its meaning depends
     on the overloaded method Connect::Process(bool), since this calls this
     callback. 
   \sa Connect::Process(bool), Query_generic::Process(bool),
     Event_generic::Process(bool) 
  */
  virtual void operator()(const Arg& p)=0;
 protected:
  /*!\brief Exclude this instance from callback list.*/
  void StopCB(){if(iCB_ID)qc.cb.Del(iCB_ID);iCB_ID=0;}
  /*!\brief Add this instance into callback list. */
  void StartCB(){if(!iCB_ID)iCB_ID=qc.cb.Add(this);}

  /*!\brief Send command to server.

   It puts the command into the output queue of the connection. It does not try
   to send it to the server; call e.g. \a Wait(false) to do so.

   \param line VTQL command line (without newline)
   \sa EscapeString(std::string)
  */
  void Start_Query(const std::string& line){qc<<line;}
 private:
  Connect& qc;
  size_t iCB_ID;
};

/*!\brief Proviedes high level output of videotext pages, as alevt does it.

 You can fill this class with the output if a PAGE ... HEX command. It should
 create the same output as the alevt programs do it (e.g. alevt-cap), since it
 uses its source code (thanks to the nice and reusable code of alevt).

\warning uses global variables, i.e. it is not thread save.
\todo Remove the global variables.
*/
class AlevtPage{ 
 public:
  /*!\brief Does not init the page */
  AlevtPage():iLC(0){};

  /*!\brief Fill the page with a line of the command output.

   Indeed, you have to call this method 26 times, i.e. 25 times for the
   25 lines of a videotext page, hexadecimal encoded from a VTQLserver and
   a 26th time for the additional information of a page_info line, encoded
   as 26th line in PAGE ... HEX mode. At this 26th call you get a true back
   to indicate, that the page has been read completly.

   If you call it more then 26 times, it starts to overwrite the old lines
   and returns true at the 52th call.
 
   \param line A line form the output of a PAGE ... HEX command.
   \return true if a full page has been read.
  */
  bool ReadLine(const std::string& line);

  /*!\brief Get a png image of the page.
   
   \param image The method writes the image to this memory block without
    checking for boundaries, so dont make it too small.
   \param comp compression factor (0: no compression ... 9: maximum compression)
   \param reveal If true, reveal hidden text.
   \return Size of the image in bytes.
  */ 
  size_t GetPng(char* image,int comp=0,bool reveal=false);

  /*!\brief Get a color ansi output of the page.
   
   Color ansi means, that this method inserts escape codes into the
   videotext page. This escape codes  will be interpreted as color attributs
   on a color terminal (e.g. this method is heavily used by console_vt.

   \param reveal If true, reveal hidden text
   \return The escaped videotext page. Every line ends with a newline character.
  */
  std::string GetAnsi(bool reveal=false);
 private:
  vt_page pg;
  size_t iLC;
};

/*!\brief Contains all the non-text information of a videotext page

 With this class you can read all information, that is in the page_info 
 (i.e. the last line of the output of page, you did get from a 
 PAGE ... ... command) line and access it easily. In addtion this class can be
 sorted in a usefill manner.
*/
struct PgInfo{
 PgInfo(){}; //!< Does not init anything.
 /*!\brief Get all field from a page_info line. 

  A page_info line is always of the end of the output of a page, if you search
  for pages/wait for events via the PAGE ... LIST|TEXT|HEX commands. 

  \param line The page_info line without newline at the end.
 */
 PgInfo(const std::string& line);

 /*!\brief Sort it by station, page, subpage */
 bool operator <(const PgInfo& p)const{
  return (iSt==p.iSt)?(
          (iPage==p.iPage)?(iSPage<p.iSPage):(iPage<p.iPage)
         ):(iSt<p.iSt);
 }

 size_t iSt, //!< station id
        iFreq, //!< frequency in kHz
        iPage, //!< page number
        iSPage; //!< subpage number
 time_t tTime; //!< system time of page recetion
 size_t iFlags; //!< some flags used by alevt

 /*!\brief list of fastlinks
  
  The pairs are (page number,subpage number). The subpage number may be 
  \c ANY_SUB, i.e. this fastlink referrs to any subpage.
 */
 std::vector<std::pair<size_t,size_t> > vFL;
};

}
#endif
