/*! \file vtql_event.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief Interface to connections to a VTQLserver, where the connection is in
     event mode.
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef VTQL_EVENT_H
#define VTQL_EVENT_H

#include <list>
#include <set>
#include "vtql_base.h"
#include "vtql_utils.h"

namespace VTQL{

/*!\brief For handling connection in event mode, you will need a connection 
 provieded by this class.

 Usually you do only care about make an instance of this class and pass it to
 the event moduls. The only non-trivial method in this class is the 
 Event_generic::Process(bool) function which tries to spot the kind of event
 arriving and counts the lines of every event.
*/
class Event_generic:public Connect{
 public:
  virtual ~Event_generic(){};

  /*! \brief Open a connection to a server.
     
   As you may suspect, this connection has to be in event mode.   
   \param file_name File name of a socket of VTQLserver.
  */
  Event_generic(const std::string& file_name):
   Connect(file_name),iLC(-1),bLM(false){};
  /*!\brief Open a connection to a server (advanced).
  
  If you have an file descriptor and it is connected to a server, you can use
  this constructor. As you may suspect, the connection has to be in event mode.   
  \param file_descriptor A file descriptor that is connected to a VTQLserver.
  \sa sock::Socket_basic::Socket_basic(int), sock::Socket_basic::operator int()
  */
  Event_generic(const sock::Socket_basic& file_descriptor):
  Connect(file_descriptor),iLC(-1),bLM(false){};
 
  /*!\brief This method gets called the event moduls.
   
   This method handles the incomming data and sends anything in the output
   queue. It calls the callbacks, where the argument pair means 
   (line number,line content), starting with line number 1.

   \param block true: blocking ; false: nonblocking i/o
   \return true if a event has been completly read.
  */
  virtual bool Process(bool block);
 private:
  int iLC;
  bool bLM;
};

/*!\brief Base class for all moduls, that handle events.
 
 The main problem we have to handle, that you do never know wether
  - your command was already passed to the server when it sent this event
  - an other command was send meanwhile to the server (this is bad, do not do
    it)
  - your command failed (i.e. a syntax error) and so you do either do not get
    any event or you get events of an old command.

 We simply compare the servers information about the command and the command
 that should be the source of the event and only call the Handle(const Arg&)
 method if anything is OK.
*/
class CEModul:public CModul{
 public:
  virtual ~CEModul(){};

  /*!\brief Send a command to a connection.

    It puts the command only into the output queue. Call Wait(bool) to try to
    send it.  Of course we do the callback stuff right, so that we do not loose 
    any event. 

    \param event_con Connection where you want to get events
    \param match Events should match this (not escaped) \a match.
    \param mode The page_output_mode for events
  */
  CEModul(Event_generic& event_con,const std::string& match,
   ShowMode mode);
  /*!\brief Wait for a incomming event of our command

   In detail, the event has to be read completly and must not be invalid
   (see \a SetInvalid()), to let this method return true. If you wait in
   blocking mode, it return always true.
   \param block switch blocking (true) or nonblocking (false) waiting.
   \return true if a matching event was recieved, false otherwise.
   */
  virtual bool Wait(bool block=true);
 protected:
  /*!\brief Handle events of the command, that was sent by this instance

   This gets only called, if the incomming event is of the specified type.
   \param p Pair of int (line number, starting with 1) and string (line content)
  */
  virtual void Handle(const Arg& p)=0;
  void SetInvalid(){bMatch=false;} //!< Mark the incomming page as invalid.
 private:
  virtual void operator()(const Arg&);
  std::string sMatch,sMode;
  bool bMatch;
};

/*!\brief Get matching events and provide advanced output methods to
 display the page.
  
 Performs a "PAGE match HEX" command. If you call the inherited Wait(bool)
 method and it has returned true, a page has been recieved and has been stored
 as \a AlevtPage and as PgInfo.
*/
class EM_PageAlevt:public CEModul{
 public:
  virtual ~EM_PageAlevt(){};

  /*!\brief Set the match for the events you want to get.

    It does not really set the match. The first try to do so will be performed
    at the first Wait(bool) call.
    \param event_con Connection where you want to get events
    \param match Events should match this (not escaped) \a match.
  */
  EM_PageAlevt(Event_generic& event_con,const std::string& match):
   CEModul(event_con,match,SM_HEX){};

  AlevtPage alevtpg; //!< the latest page as \a AlevtPage
  PgInfo pginfo; //!< the latest page info
 private:
  virtual void Handle(const Arg& p);
};


}
#endif
