//content       pharsing a VTQL command
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef VTQL_PHARSE_H
#define VTQL_PHARSE_H

#include <string>
#include <vector>
#include <list>
#include <regex.h>
#include "error_base.h"

namespace VTQL{

/*!\brief Parse a VTQL command line 
 
 This class is mainly used in the server itself, but maybe it is handy to
 know about it in a client application.
*/
class Parse{
 public:
  /*!\brief Parse a new line and delete the old results.
   
   \param line New command line, without a newline character.
  */
  void NewString(const std::string& line);
  /*!\brief Get the arbitrary number of the command

   \return The index number in the \a VTQL::cmAllCmds[] array.
  */
  int GetCom()const;
 
  /*!\brief Get an integer command argument in decimal base.
   
   Interpret a field as integer in decimal writing and specify a range for it.

   \param field Field number. E.g. in "STATION 123 ADD" the field "123" has the
     field number 1.
   \param min Minmum value allowed.
   \param max Maximum value allowed.

   \return The value of the field, in the example above: 123 
  */
  int Int10(int field,int min=INT_MIN,int max=INT_MAX)const;
  
  /*!\brief Get an frequency argument
    
   Interpret a field as frequency argument (e.g. '12345' (kHz) or
    'europe_west::E2~m3').
   
   \param field Field number. E.g. in "FREQ europe_west::E2~m3 ZAP" the field
    "europe_west::E2~m3" has the field number 1.

   \return The frequency in kHz of the field, in the example above: 48063 
  */
  int Freq(int field)const;

  /*!\brief Get an string argument

   \param field Field number. E.g. in "STATION 123 NAME 'it''s a '''" the field
    "europe_west" has the field number 3
   \return The field as string (without the "'"s), in the example above
    "it's a '"
  */
  std::string String(int field)const;

  /*!\brief Get an \a ShowMode or \a UserMode argument

   \param ModeT This is either \a UserMode or \a ShowMode
   \param field Field Number. E.g. in "SOCKET 123 MODE query" the field "query"
    has the field number 3
   \return The \c enum value of the field, in the example above
    \a UM_QUERY
  */
  template<typename ModeT>
  ModeT DecMode(int field)const;
 private:
  std::vector<std::string> slWords;
};

/*!\brief Parse matches within a VTQL command and test input lines for matches
 
 This handles the monsters called match (read VTQL syntax documentation!), such
 as "-98.unknown,europe_west::E2.100,333..2-/linux.+OS"  . But is is quite
 general and may be used for simular queries. The interface is wired, since it
 is quite critical to the speed of the server, but if you really need VTQL
 compatible match parsing this may be usfull, anyway.
*/
class Region_basic{
 public:
  virtual ~Region_basic();
  /*!\warn No implementation of the copy constructor.*/
  Region_basic(const Region_basic&);

  /*!\brief Construct and define the syntax
   
   The fields separate with "." may have a different base, such as decimal (10)
   or hexadecimal (16). There is a special base 0 for frequency fields. For
   any entry in the vector you have a field in your query. The VTQL standard
   is {10,0,16,16,10} (station_id,freuquency,page_nr,subpage_nr,line_nr).
   \param bases The base for each field of the match.
  */ 
  Region_basic(const std::vector<size_t>& bases);

  /*!\brief Parse a new match string

    Cleans all the data of the last query and parses a new match.

   \param match The match string, that should be parsed.
   \param search Search modes allowed.
  */
  virtual void NewString(std::string match,const std::string& search="/|\\");


  //write VT_WIDTH chars into char* and terminate with '\0'
  struct PageData{
   virtual ~PageData(){};
   virtual void GetLine(size_t,char*)const=0;
  };

  bool MatchTxt(const PageData&)const;
  bool IsIn(int,int)const;
  bool HasTextSearch()const{return cpFind;}
 protected:
  typedef std::list<std::pair<int,int> > SingleRange;
  typedef std::vector<SingleRange> RangeVec;
  const std::vector<size_t> vBases;
  RangeVec rvAll;
 private:
  void Remove_Regexp();
  regex_t* rSearch;
  char* cpFind;
  bool bGlobSearch;
  void ExtractRegex(std::string&,const std::string&);
 
  mutable char* cpTT;
};
}

namespace Error{
/*!\brief An VTQL syntax error occured
 
 This one gets thrown if an syntax error in VTQL occured while parsing a VTQL
 command or a part of it.
*/
struct Syntax:Runtime{ 
 virtual ~Syntax(){};
  /*!\brief Construct an syntax error.
   \param fu_name Code position of error (ususally \a METH_NAME).
   \param desc Error description.
  */
 Syntax(const std::string& fu_name,const std::string& desc):
  Runtime(fu_name,desc){};
 //! Get an pretty error string.
 virtual operator std::string(){return std::string("syntax error")+
#ifdef DEBUG
  " in "+sFunc+
#endif
  ": "+sWhat;}
};
}

#endif
