/*! \file vtql_query.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief Interface to connections to a VTQLserver, where the connection is in
     query or configuration mode.
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef VTQL_QUERY_H
#define VTQL_QUERY_H

#include <list>
#include "vtql_base.h"
#include "vtql_utils.h"

namespace VTQL{

/*!\brief For handling connection in query or configuration mode, you will
  need a connection provieded by this class.

 Usually you do only care about make an instance of this class and pass it to
 the query moduls. The only non-trivial aspect of this class is, that it
 gives any command an ID and you get the ID of an command output, so that you
 can easily identify the output of your command. This is done by the
 \a CQModul class.
*/
class Query_generic:public Connect{
 public:
  virtual ~Query_generic(){};

  /*!\brief Open a connection to a server.

   As you may suspect, this connection has to be either in query or connection
    mode.   
   \param file_name File name of a socket of VTQLserver
  */
  Query_generic(const std::string& file_name);
  /*!\brief Open a connection to a server (advanced).
   
   If you have an file descriptor and it is connected to a server, you can use
   this constructor.  As you may suspect, this connection has to be either in
   query or connection mode.   
   \param file_descriptor A file descriptor that is connected to a VTQLserver.
   \sa sock::Socket_basic::Socket_basic(int), sock::Socket_basic::operator int()
  */
  Query_generic(const sock::Socket_basic& s);
 
  /*!\brief  This method gets called the query moduls.
   
   This method handles the incomming data and sends anything in the output 
   queue. It calls the callback, where the argument pair is
    - (>0,line) A normal line (not a error message and not SUCCESS or VOID or
     ERROR).
     -# the line number, stating with 1
     -# the line itself (without newline character)
    - (0,"") The Command was successfully completed.
    - (-1,"") The Command was void. 
    - (-2,Error) Error on command execution.
     -# -2
     -# error string

   \param block true: blocking ; false: nonblocking i/o
   \return true if a command was completed read
  */
  virtual bool Process(bool block);
  virtual Connect& operator <<(const std::string& s)
   {++iWriteID;return Connect::operator <<(s);}

  /*!\brief Get the command ID of the latest command, you put with 
   \a operator<<(const std::string)*/
  size_t LatestWriteID()const{return iWriteID;}
  /*!\brief  Get the command ID of the latest processed command
   
   It is already set before the callbacks ar called, so you can find out, 
   which ID the current line has.
  */
  size_t LatestReadID()const{return iReadID;}
 private:
  int iLC;
  size_t iReadID,iWriteID;
  std::string sBuff;
  bool bBuff;
};

/*!\brief Handle errors at command execution

 This function is very handy if you want to know about the status of the
 execution of a command, such as VTQL syntax errors. Since this is a flag
 if the CQModul, the server errors are handled autmatically and if you inherit
 from CQModul and you have a unified design.
*/
class QMError_flag{
 public:
  /*!\brief Init this object to a WriteID and set QME_RUNNING flag

    \param id WriteID you want to watch.
    \sa Query_generic::LatestWriteID()
  */
  QMError_flag(size_t id=0):Err(QME_RUNNING),sErr("Running..."),iID(id){};
  /*!\brief Set the WriteID
   
    \param id WriteID you want to watch.
    \sa Query_generic::LatestWriteID()
  */
  void SetID(size_t i){iID=i;}
  /*!\brief Check a callback argument for messages
   
    This class expect of course to be attaced to a \a Query_generic instance.
    This method is called by the callback handler of \a CQModul.
    \param p The callback argument
  */
  void Handle(const Query_generic::CallB::CBFunct::Arg& p); 

  /*!\brief Acutal state of the command.
  
   If you Wait(bool) did return \a true, you can get the success status of the
   command.  If you do not know about that return value, you can check for the
   \a QME_RUNNING flag.
  */
  enum {QME_RUNNING=-1, //!< the query is not completed yet
        QME_SUCCESS,    //!< the last query returned sucessfully
        QME_VOID,       //!< the last query was a void query (empty line)
        QME_ERROR       //!< an error \a sMsg occured on last query
       } Err;
  
  std::string sErr; //!< almost the same as the \a Err flag, but as string
  std::string sMsg; /*!< If QME_ERROR is set, this containts the error message
   of the server, "" otherwise*/
 private:
  size_t iID;
};

/*!\brief Base class for all moduls, that handle queries.
 
 This class tries to abstrct form all the server error handling and makes sure,
 that a child class only gets a callback, if the server answers successfully to
 the query of the child. 
*/
class CQModul:public CModul{
 public:
  virtual ~CQModul(){};

  /*!\brief Send a command to a connection.
     
   It puts the command only into the output queue. Call Wait(bool) to try to
   send it. Of cource we care about the ID stuff.
   \param query_con Connection where you want to query.
   \param command The command you want to be executed, without newline.
  */
  CQModul(Query_generic& query_con,const std::string& command);

  /*!\brief wait for this command to complete

   Call this function to wait until this command has completed,
   blocking or nonblocking.
   \param b if false, test if it is completed and return nonblocking, if true,
         wait for completion
   \return true if the command was recieved, false otherwise
  */
  virtual bool Wait(bool=true);

  QMError_flag error_flag; //!< Command status flag
 protected:
  /*!\brief Handle the command output
   
   This mathod gets called, if the server responds to your query sucessfully.
   You get a call for every output line.

   \param p Pair of int (line number, starting with 1) and string (line content)
  */
  virtual void Handle(const Arg& p)=0;
 private:
  virtual void operator()(const Arg& p);
  Query_generic& qgCon;
  size_t iID;
};

/*!\brief Get a list of PgInfo from a search
 
  Performs a "PAGE match LIST" command and stores the matching page numbers in
  a list.
*/
class QM_PageL:public CQModul{
 public:
  virtual ~QM_PageL(){};

  /*!\brief query for a page list
    \param _qc query connection you want this modul to be attached 
    \param s the (not escaped) match, you want to search for
  */
  QM_PageL(Query_generic& _qc,const std::string& s):
   CQModul(_qc,std::string("PAGE ")+EscapeString(s)+" LIST"){};
  
  //@{ \name Set of Page Information
  typedef std::list<PgInfo>::iterator iter; //!< iterator to list
  std::list<PgInfo> sPg; //!< list of page informations
  //@}
  private:
   virtual void Handle(const Arg& p){sPg.push_back(PgInfo(p.second));}
};

/*!\brief Get a list of \a AlevtPage from a search
 
  Performs a "PAGE match HEX" command and stores the matching page numbers in
  a list as \a AlevtPage.
*/
class QM_PageAlevt:public CQModul{
 public:
  virtual ~QM_PageAlevt(){};

  /*!\brief query for a AlevtPage list
    \param _qc query connection you want this modul to be attached 
    \param s the (not escaped) match, you want to search for
    \param i limit the number of results (\c 0 : no limit)
  */
  QM_PageAlevt(Query_generic& _qc,const std::string& s,size_t i=0):
   CQModul(_qc,std::string("PAGE ")+EscapeString(s)+" HEX"),iLim(i){};

  //@{ \name List of \a AlevtPage
  //! iterator to list
  typedef std::list<std::pair<PgInfo,AlevtPage> >::iterator iter;
  std::list<std::pair<PgInfo,AlevtPage> > li; //!< list of pages 
  //@}
 private:
  virtual void Handle(const Arg&);

  AlevtPage pg;
  size_t iLim;
};

/*!\brief Get a list of stations
 
  Performs a "SHOW STATION" command and stores the station info in a list
*/
class QM_Stations:public CQModul{
 public:
  virtual ~QM_Stations(){};

  /*!\brief query for a list of stations
    \param _qc query connection you want this modul to be attached 
  */
  QM_Stations(Query_generic& _qc):CQModul(_qc,"SHOW STATION"){};

  //@{ \name List of Station Info
  //! station information
  struct StInfo{
   std::string sName;    //!< name of station
   std::string sMatch;   //!< match expression for station
   size_t iTimeout;      //!< timeout for station pages
   size_t iPageC;        //!< number of pages in station
   time_t tLatest;       //!< time of latest page in station
  };

  typedef std::map<size_t,StInfo>::iterator iter; //!< iterator to map
  std::map<size_t,StInfo> sm;  //!< map of station id --> station info
  //@}
 private:
   virtual void Handle(const Arg&);
};

}
#endif
